package com.covas.Resources;

import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;
import java.util.ArrayList;
import java.util.Collection;


import javax.annotation.security.RolesAllowed;
import javax.enterprise.context.Conversation;
import javax.inject.Inject;
import javax.transaction.Transactional;
import javax.ws.rs.core.SecurityContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.CookieParam;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.PATCH;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import com.covas.Entity.ConversationEntity;
import com.covas.Entity.FriendEntity;
import com.covas.Entity.GroupEntity;
import com.covas.Entity.UsersEntity;

import io.quarkus.panache.common.Page;

import org.bson.types.ObjectId;
import org.eclipse.microprofile.jwt.Claims;
import org.eclipse.microprofile.jwt.JsonWebToken;
import org.jboss.logging.Logger;
import org.postgresql.shaded.com.ongres.scram.common.bouncycastle.base64.Base64;

@Produces(MediaType.APPLICATION_JSON)
@Path("conversation")
public class ConversationRessources {
    private static final Logger LOGGER = Logger.getLogger(UsersRessources.class);
    @Inject
    JsonWebToken jwt;

    /// Function
    private Boolean checkUserCookie(String userCookie, UsersEntity users) {
        if ((userCookie == null) || (users == null)) {
            return false;
        }
        String name = new String(Base64.decode(userCookie), StandardCharsets.UTF_8);
        if (!name.equals(users.pseudo) && (users.status != 1)) {
            return false;
        }
        return true;
    }

    private Response.Status getResponseCheck(SecurityContext ctx, String userCookie, UsersEntity users) {
        if (!ctx.getUserPrincipal().getName().equals(jwt.getName())) {
            return Response.Status.INTERNAL_SERVER_ERROR;
        }
        if (!checkUserCookie(userCookie, users)) {
            return Response.Status.FORBIDDEN;
        }
        return Response.Status.OK;
    }

    /// Appel HTTP


    /// GET
    @GET
    @RolesAllowed("Admin")
    public Response getFriends(@CookieParam("user") String userCookie, @Context SecurityContext ctx, 
    @QueryParam("page") Integer page, @QueryParam("nbPages") Integer nbPages) {
        if(nbPages == null){
            nbPages = 20;
        }
        if(page == null){
            page = 0;
        }
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status statusHttp = getResponseCheck(ctx, userCookie, user);
        Response responseHttp = Response.status(statusHttp).build();
        if (statusHttp.equals(Response.Status.OK)) {
            List<ConversationEntity> listConversations = ConversationEntity.findAll().page(Page.of(page, nbPages)).list();
            responseHttp = Response.ok(listConversations).build();

        }
        return responseHttp;
    }

    @GET
    @RolesAllowed("Admin")
    @Path("count")
    public Response getCount(@CookieParam("user") String userCookie, @Context SecurityContext ctx){
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        Response responseHttp = Response.status(status).build();
        if (status.equals(Response.Status.OK)){
            responseHttp = Response.ok(ConversationEntity.count()).build();
        }
        return responseHttp;
    }

    @GET
    @RolesAllowed({"Admin", "User"})
    @Path("{id}")
    public Response getSingleFriend(@PathParam("id") String id, @CookieParam("user") String userCookie,
            @Context SecurityContext ctx) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        Response responseHttp = Response.status(status).build();
        if (status.equals(Response.Status.OK)) {
            ConversationEntity conversation = ConversationEntity.findById(new ObjectId(id));
            responseHttp = Response.status(Response.Status.NOT_FOUND).build();
            if (conversation != null) {
                responseHttp = Response.ok(conversation).build();
            }

        }
        return responseHttp;
    }


    
    // PUT
    @PUT
    @Consumes(MediaType.APPLICATION_JSON)
    @RolesAllowed({"Admin", "User"})
    @Path("{id}")
    @Transactional
    public Response createFriend(@Context SecurityContext ctx, @CookieParam("user") String userCookie, @PathParam("id") String id, ConversationEntity conversation) {

            UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
            UsersEntity user = UsersEntity.findById(kid);
            Response.Status status = getResponseCheck(ctx, userCookie, user);
            ConversationEntity newConversation = new ConversationEntity();
            if (status.equals(Response.Status.OK)) {
                newConversation.users = conversation.users;
                newConversation.created_at = LocalDateTime.now();
                newConversation.updated_at = LocalDateTime.now();
                newConversation.persist();
                status = Response.Status.CREATED;
                
            }            
        return Response.status(status).entity(newConversation).build();
    }


    // DELETE
    @DELETE
    @Path("{id}")
    @RolesAllowed({"Admin", "User"})
    @Transactional
    public Response changeStatusSingleFriendToDelete(@Context SecurityContext ctx, @CookieParam("user") String userCookie,
            @PathParam("id") String id) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);
        if (status.equals(Response.Status.OK)) {
            ConversationEntity singleConversation = ConversationEntity.find("id", new ObjectId(id)).firstResult();
            if (singleConversation == null) {
                status = Response.Status.NOT_FOUND;
            } else {
                singleConversation.status = -1;
                singleConversation.updated_at = LocalDateTime.now();
                singleConversation.deleted_at = LocalDateTime.now();
                singleConversation.persist();
            }
        }
        return Response.status(status).build();
    }

    

    // PATCH
    @PATCH
    @RolesAllowed({"Admin", "User"})
    @Consumes(MediaType.APPLICATION_JSON)
    @Transactional
    @Path("{id}")
    public Response updateFriend(@Context SecurityContext ctx, @CookieParam("user") String userCookie,
    ConversationEntity conversation, @PathParam("id") String id) {
        UUID kid = UUID.fromString(jwt.getClaim(Claims.kid));
        UsersEntity user = UsersEntity.findById(kid);
        Response.Status status = getResponseCheck(ctx, userCookie, user);

        if (status.equals(Response.Status.OK)) {

            ConversationEntity conversationOrig = ConversationEntity.findById(new ObjectId(id));
            if (conversationOrig == null) {
                status = Response.Status.NOT_FOUND;
            } else {
                conversationOrig.users = conversation.users;

                
                conversationOrig.updated_at = LocalDateTime.now();
                
                if(conversation.status == 1){
                    conversationOrig.deleted_at = null;
                    conversationOrig.status = 1;
                }
                conversationOrig.persist();
                
            }
        }
        return Response.status(status).build();
    }
}